# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division

#! List of available HTTP/1.1 connectors.
http = ('bottle', 'flask', 'django',)

#! List of available ORM connectors.
model = ('django', 'sqlalchemy')
